--------------------------------------
-- Namespaces
--------------------------------------
local _, core = ...
core.AltVaultGuildVault = {}

local AltVault = core.AltVault
local AltVaultGuildVault = core.AltVaultGuildVault

local ITEMS_PER_ROW = 8
local ICON_HEIGHT = 44
local CONTAINER_PADDING = 68
local TAB_SLOTS = 98
local MAX_NUM_TABS = 8

function AltVaultGuildVaultFrame_OnLoad(self)
    local scrollFrame = AltGuildVaultScrollFrame
    local scrollChild = AltGuildVaultScrollFrameScrollChild
    local containerHeight = CONTAINER_PADDING + (ceil(TAB_SLOTS / ITEMS_PER_ROW) * ICON_HEIGHT)
    scrollFrame:SetScrollChild(scrollChild)
    scrollChild:SetWidth(scrollFrame:GetWidth())

    -- create all the guild vault tabs
    for tabID = 1, MAX_NUM_TABS do
        local tabContainer = CreateFrame("Frame", "$parentAltGuildVaultContainer"..tabID, scrollChild, "AltBagContainerTemplate")
        tabContainer:SetHeight(containerHeight)
        tabContainer:SetID(tabID)
        if tabID ~= 1 then
            tabContainer:SetPoint("TOPLEFT", "$parentAltGuildVaultContainer"..tabID - 1, "BOTTOMLEFT")
        end
        for slotID = 1, TAB_SLOTS do
            local itemFrame = CreateFrame("ItemButton", "$parentAltVaultTabItem"..slotID, tabContainer, "AltVaultBagItemTemplate")
            itemFrame:SetID(slotID)
            itemFrame:SetScript("OnEnter", AltVaultTabSlotButton_OnEnter)
            itemFrame:SetScript("OnClick", AltVaultTabSlotButton_OnModifiedClick)
            local rowID = ceil(slotID / ITEMS_PER_ROW)
            if rowID > 1 and (slotID - (ITEMS_PER_ROW * (rowID - 1)) == 1) then
                local offsetID = slotID - ITEMS_PER_ROW
                itemFrame:SetPoint("TOPLEFT", '$parentAltVaultTabItem'..offsetID, "BOTTOMLEFT", 0, -4)
            elseif slotID ~= 1 then
                itemFrame:SetPoint("TOPLEFT", '$parentAltVaultTabItem'..slotID-1, "TOPRIGHT", 4, 0)
            end
        end
        tabContainer:Show()
    end
end

function AltVaultGuildVaultFrame_OnShow(self)
	UpdateGBankBags()
end

function UpdateGBankBags()
    local gbankBags = AltVaultDB.characters[CharacterJournal.activeAlt].gbank
    local scrollChild = AltGuildVaultScrollFrameScrollChild
    local gbankFrames = { scrollChild:GetChildren() }
    local isSearching = CharacterJournal.searchText ~= nil and CharacterJournal.searchText ~= "" and AltVault:TrimString(CharacterJournal.searchText) ~= ""
    if gbankBags ~= nil then 
        AltGuildVaultNotAvailable:Hide()
        scrollChild:SetHeight(GetTotalVaultHeight() + 20)
        for tabID, tabContainer in ipairs(gbankFrames) do
            local tabData = gbankBags["tab-"..tabID]
            local tabHeight
            if (tabData ~= nil and tabData.items ~= nil and #tabData.items ~= 0) or (tabData ~= nil and tabData.freeSlots == TAB_SLOTS) then
                tabHeight = CONTAINER_PADDING + (ceil(TAB_SLOTS / ITEMS_PER_ROW) * ICON_HEIGHT)
            else
                tabHeight = 100; -- calculate locked containers
            end
            tabContainer:SetHeight(tabHeight)
            tabContainer.BagTitle.AltBagIcon:SetVertexColor(1.0,1.0,1.0)
            tabContainer.BagLocked:Hide()

            tabContainer.BagTitle.AltBagTitle:SetTextColor(GetItemQualityColor(1))
            local tabItems = { tabContainer:GetChildren() }
            table.remove(tabItems, 2)
            table.remove(tabItems, 1)

            -- tab-number doesn't exist
            if tabData == nil or tabData.name == nil then
                tabContainer.BagTitle.AltBagTitle:SetText("Tab"..tabID)
                tabContainer.BagTitle.AltBagIcon:SetVertexColor(1.0,0.1,0.1)
                tabContainer.BagTitle.AltBagCapacity:SetText("0/0")
                for slotID, itemFrame in ipairs(tabItems) do
                    itemFrame:Hide()
                end
                tabContainer.BagLocked:Show()
            else
                tabContainer.BagTitle.AltBagTitle:SetText(tabData.name)
                tabContainer.BagTitle.AltBagIcon:SetTexture(tabData.icon)
                tabContainer.BagTitle.AltBagCapacity:SetText(TAB_SLOTS - tabData.freeSlots..'/'..TAB_SLOTS)
                for slotID, itemFrame in ipairs(tabItems) do
                    if (#tabData.items ~= 0 or tabData.freeSlots == TAB_SLOTS) and slotID <= TAB_SLOTS then
                        local rowID = ceil(slotID / ITEMS_PER_ROW)
                        if rowID > 1 and (slotID - (ITEMS_PER_ROW * (rowID - 1)) == 1) then
                            local offsetID = slotID - ITEMS_PER_ROW
                            itemFrame:SetPoint("TOPLEFT", '$parentAltVaultTabItem'..offsetID, "BOTTOMLEFT", 0, -4)
                        elseif slotID ~= 1 then
                            itemFrame:SetPoint("TOPLEFT", '$parentAltVaultTabItem'..slotID-1, "TOPRIGHT", 4, 0)
                        end
                        local texture = _G[itemFrame:GetName().."IconTexture"]
                        local slotItem = tabData.items[tonumber(slotID)]
                        local itemCount = tabData.counts[tonumber(slotID)]
                        if slotItem ~= nil and slotItem.link ~= nil then
                            texture:SetTexture(slotItem.icon)
                            SetItemButtonQuality(itemFrame, slotItem.rarity, 1)
                            if itemCount ~= nil then
                                SetItemButtonCount(itemFrame, itemCount)
                            else
                                SetItemButtonCount(itemFrame, nil)
                            end
                        else
                            texture:SetTexture("Interface\\PaperDoll\\UI-Backpack-EmptySlot")
                            SetItemButtonQuality(itemFrame, 1, 1)
                            SetItemButtonCount(itemFrame, nil)
                        end
                        if isSearching then
                            local matchesSearch = AltVault:ItemInSearch(slotItem.searchName)
                            if matchesSearch == false then
                                SetItemButtonTextureVertexColor(itemFrame, 0.1,0.1,0.1);
                            else
                                SetItemButtonTextureVertexColor(itemFrame, 1,1,1);
                            end
                        else
                            SetItemButtonTextureVertexColor(itemFrame, 1,1,1);
                        end
                        itemFrame:Show()
                    else
                        itemFrame:Hide()
                    end
                end
                tabContainer:Show()
            end
        end
    else
        AltGuildVaultNotAvailable:Show()
        scrollChild:SetHeight(0)
        for tabID, tabContainer in ipairs(gbankFrames) do
            tabContainer:Hide()
        end
    end
end

function GetTotalVaultHeight()
    local gbank = AltVaultDB.characters[CharacterJournal.activeAlt].gbank
    local totalHeight = 0
    for key, container in pairs(gbank) do
        local numRows = ceil(TAB_SLOTS / ITEMS_PER_ROW)
        if (container.items ~= nil and #container.items ~= 0) or container.freeSlots == TAB_SLOTS then
            totalHeight = totalHeight + (CONTAINER_PADDING + (numRows * ICON_HEIGHT))
        else
            totalHeight = totalHeight + 100
        end
    end
    return totalHeight
end

function AltGuildVaultScroll_OnScroll(self, delta)
    local newValue = self:GetVerticalScroll() - (delta * 50)
    if newValue < 0 then
        newValue = 0
    elseif newValue > self:GetVerticalScrollRange() then
        newValue = self:GetVerticalScrollRange()
    end
    self:SetVerticalScroll(newValue)
end

function AltVaultTabSlotButton_OnEnter(self)
    GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
    local parentID = self:GetParent():GetID()
    local itemData = AltVaultDB.characters[CharacterJournal.activeAlt].gbank["tab-"..parentID].items[tonumber(self:GetID())]
    if itemData and itemData.link then
        local isBattlePet = itemData.link:match("|Hbattlepet:")
        if isBattlePet == nil then
            GameTooltip:SetHyperlink(itemData.link)
        else
            local _, speciesID, level, breedQuality, maxHealth, power, speed, battlePetID = strsplit(":", itemData.link)
            local name = select(8, C_PetJournal.GetPetInfoByPetID(battlePetID))
            BattlePetToolTip_Show(
                tonumber(speciesID),
                tonumber(level),
                tonumber(breedQuality),
                tonumber(maxHealth),
                tonumber(power),
                tonumber(speed),
                name
            );
        end
        CursorUpdate(self);
	end
end

function AltVaultTabSlotButton_OnModifiedClick(self, button)
    if ( IsModifiedClick() ) then
        local parentID = self:GetParent():GetID()
        local itemData = AltVaultDB.characters[CharacterJournal.activeAlt].gbank["tab-"..parentID].items[tonumber(self:GetID())]
        local _, itemHyperLink = GetItemInfo(itemData.link)
        if ( itemData and itemHyperLink ) then
            HandleModifiedItemClick(itemHyperLink)	
        end
    end
end